#include <string.h>

#include "owbasic.h"
#include "shared.h"
#include "graphic.h"
#include "ostring.h"
#include "file.h"

#ifdef __HITACHI__
#undef tolower
#endif

static const byte FAR kind[16]={
  FILE_KIND_TEXT,FILE_KIND_TEXT,FILE_KIND_TEXT,FILE_KIND_TEXT,
  FILE_KIND_TEXT,FILE_KIND_BIN, FILE_KIND_BIN, FILE_KIND_TEXT,
  FILE_KIND_TEXT,FILE_KIND_TEXT,FILE_KIND_BIN, FILE_KIND_BIN,
  FILE_KIND_TEXT,FILE_KIND_TEXT,FILE_KIND_TEXT,FILE_KIND_TEXT};

FILE_INFO finf;

/* transform OWB submode -> PVOS submode + secretinfo */
#ifdef FAST_MACRO
#define Sub(sub)  (sub & 0x0F)
#define Info(sub) (((sub) > 15) ? FILE_SECRET_MODE : FILE_OPEN_MODE)
#else
int Sub(int sub) { return sub & 0x0F; }

int Info(int sub)
{
  if (sub>15) return FILE_SECRET_MODE;
  return FILE_OPEN_MODE;
}
#endif

/* Fill file datastructures */
void SetFP (int mode,int sub,word fp)
{
  fb.fsb_main_entry_ = mode << 4;	/* Main */
  fb.fsb_sub_entry_  = Sub (sub);	/* Sub */
  fb.fsb_scrt_info_  = Info (sub);	/* Open Mode */

  fb.fsb_todo_chek_ = 0xFF;
  fb.fsb_todo_rank_ = 0xFF;

  finf.fp      = fp;
  finf.kind    = kind[mode];
}

/*---------basic file operations -----------------*/
bool fileread(int mode,int sub,word fp)
{
  SetFP(mode,sub,fp);
  return LibFileReadEx(&fb,&finf,3*1024/FILE_BLOCK_SIZE);
}

word filewrite(int mode,int sub, word fp)
{
  SetFP(mode,sub,fp);
  if (LibFileWrite(&fb,&finf)!=TRUE) return 0xffff;
  return finf.fp;
}

bool fileremove(int mode,int sub,word fp)
{
  SetFP(mode,sub,fp);
  return LibFileRemove(&fb,&finf);
}

bool filemove(int mode,int sub,word fp,word fpz)
{
  SetFP(mode,sub,fp);
  return LibFileExch(&fb,&finf,fpz);
}

word filenext(int mode,int sub,word fp)
{
  SetFP(mode,sub,fp);
  if (!LibFileFindNext(&fb,&finf,0x00)) return 0xffff;
  return finf.fp;
}

word fileprev(int mode,int sub,word fp)
{
  SetFP(mode,sub,fp);
  if (!LibFileFindPrev(&fb,&finf,0x00)) return 0xffff;
  return finf.fp;
}

/* transform number to fp */
word get_file_fp(int mode,int sub, int nr)
{
  int found;
  if (nr==-1) return 0xffff;

  SetFP(mode,sub,0xffff);

  while ((found=LibFileFindNext(&fb,&finf,0x00))&&(nr>0)) nr--;
  if (!found) return 0xffff;            /* failed */
  return finf.fp;
}

bool load_file(int mode,int sub,int nr)
{
  word fp;

  fp=get_file_fp(mode,sub,nr);
  if (fp==0xffff) return FALSE;

  return fileread(mode,sub,fp);
}

bool save_file(int mode,int sub, int nr)
{
  SetFP(mode,sub,get_file_fp(mode,sub,nr));
  return LibFileWrite(&fb,&finf);
}

/* ------------------- MEMO  ----------------- */
int fb2array(int addr,int size,int *total)
{
  /* transfer filebuffer to basic string array */
  int l;
  int rc;

  l=0;
  *total=0;
  fbinit();
  spush1();
  fbstring(STOS,&rc,1); /* read from buffer */
  while ((l<size)&&(rc==0))
    {
      (*total)++;        /* count line */
      spop(addr); addr++; /* store string to array element */
      spush1();
      fbstring(STOS,&rc,1); /* next string */
      l++;
    }
  spop1();

  while(l<size)
    {
      /* clear remaing array elements */
      spush1(); STOS[0]=0; spop(addr);
      addr++; l++;
    }

  spush1();
  while (rc==0)
    {
      (*total)++;
      fbstring(STOS,&rc,1);
    }
  spop1();
  return 0;
}

int array2fb(int addr,int size,int anz)
{
  int i, rc;
  if (anz==-1) anz=size; /* store all elements */
  else
    {
      /* store anz elements */
      if (anz>size) return 38; /* valid number ?*/
      if (anz<1) return 38; /* valid number ?*/
    }
  fbinit();
  i=0;
  while (i<anz)
    {
      spush(addr); addr++;
      rc=fbstore(STOS,1);
      spop1();
      if (rc!=0) return rc;
      i++;
    }
  return 0;
}

int loadmemofp(int addr,int anz, int sub, word fp,int *total)
{
  *total=0;
  if (!fileread(3,sub,fp)) return 39;
  return fb2array(addr, anz, total);
}

int savememofp(int addr,int size, int sub, word *fp, int anz)
{
  int rc;
  rc=array2fb(addr,size,anz);
  if (rc!=0) return rc;
  *fp=filewrite(3,sub,*fp);
  if (*fp==0xffff) return 39;
  return 0;
}

int loadmemo(int addr,int size,int sub,int nr,int *total)
{
  word fp;

  fp=get_file_fp(3,sub,nr);
  if (fp==0xffff) return 61;
  return loadmemofp(addr,size,sub,fp,total);
}

int savememo(int addr,int size,int sub,int nr,int anz)
{
  word fp;
  if (nr==-1) { fp=0xffff; }
  else
    {
      fp=get_file_fp(3,sub,nr);
      if (fp==0xffff) return 61;
    }
  return savememofp(addr,size,sub,&fp,anz);
}

int filefind (int mode, int submode, const uchar* name, word* fp)
{
  uchar mname[MAXSTRLEN]; /* name of file */
  uchar sname[MAXSTRLEN]; /* name to search for */
  bool found = FALSE;
  int ic;
  byte* bp;

  strcpy ((char*) sname, (const char*) name);
  strlower (sname);

  *fp = filenext (mode, submode, 0xFFFF); /* find first */
  while ((!found) && (*fp != 0xFFFF)) /* while file exist and not found */
  {
    if (fileread (mode, submode, *fp)) /* read memo */
    {
      ic = 0;
      bp = FB;
      while (*bp == ' ')
        ++bp;
      while ((*bp != 0x0D) && (*bp != 0) && (ic < MAXSTRLEN - 1))
        mname[ic++] = tolower (*(bp++)); /* normalize name */
      mname[ic] = 0;
      trim (mname);
      found = (strcmp ((const char*) mname, (const char*) sname) == 0); /* check name */
    }
    if (!found)
      *fp = filenext (mode, submode, *fp);
  }
  return (found);
}

bool load_memo_named(int sub,const uchar *sign,const uchar *name,word *fp)
{
  uchar mname[MAXSTRLEN];
  uchar* pm = mname;

  if (sign)
  {
    strcpy ((char*) pm, (char*) sign);
    pm += 2;
  }
  strcpy ((char*) pm, (char*) name);
  filefind (3, sub, mname, fp);

#if FALSE
  byte *p=fb.fbuf.text.fsb_text_buf_;
  uchar mname[NAMELEN]; /* name of memo */
  uchar sname[NAMELEN]; /* name to search for */
  int ic, sc;
  bool rc;
  bool found;
  strcpy_name(sname,name);
  strlower(sname); /* normalize search name */
#if FALSE
  if (sub<6)
#endif
    {
      *fp=filenext(3,sub,0xffff); /* find first */
      found=FALSE;
      while ((!found)&&(*fp!=0xffff)) /* while file exist and not found */
  {
    rc=fileread(3,sub,*fp); /* read memo */
    if (rc)
      {
        sc=0;
        if (sign!=NULL)
    {
      found=strncmp((const char*)sign,(const char *)p,2)==0; /* check signature */
      sc=2; /* position to continue */
    }
        else found=TRUE;
        if (found)
    {
      ic=0;
      while (p[sc]==' ') sc++;
      while ((p[sc]!=0x0d)&&(p[sc]!=0)&&(ic<NAMELEN-1))
        mname[ic++]=tolower(p[sc++]); /* normalize name */
      mname[ic]=0;
      trim(mname);
      found=(strncmp((const char*)mname,(const char*)sname,NAMELEN)==0); /* check name */
    }
      }
    if (!found) *fp=filenext(3,sub,*fp);
  }
      return found;
    }
#if FALSE
  else
    {
      strcpy((char*)mname,"OWBasic,");
      if (sign!=NULL) strncat((char*)mname,(char*)sign,2);
      strcat((char*)mname,(char*)sname);
      sfLoadRecord(mname,2049,fb.fbuf.text.fsb_text_buf_);
    }
#endif
#endif
}

int loadmemonamed(int addr,int size,
      int sub,const uchar *name,const uchar *sig,
      int *total, word *fp)
{
  if (!load_memo_named(sub,sig,name,fp)) return 61;
  return fb2array(addr, size, total);
}

/*-------- Quickmemo -------------------------------------*/

int loadqmfp(int addr,int size,int sub,word fp)
{
  int i;
#ifdef __HITACHI__
  short val;
  int val1, val2;
#else
  int val;
#endif
  if (size<1301) return 30;
  xdata[addr]=0; /* icon invalid */
  if (fp==0xffff) return 61;
  if (!fileread(11,sub,fp)) return 39;
  xdata[addr]=156; xdata[addr+1]=130;
  fbbinit();
  for (i=0; i<1300; i++)
    {
#ifdef LSI_C
      fbbint(&val);
#endif
#ifdef __HITACHI__
      fbbbyte(&val1);
      fbbbyte(&val2);
      val=val1 + (val2 << 8);
#endif
      xdata[addr+2+i]=val;
    }
  return 0;
}

int loadqmhash(int addr,int size,int sub,int crc1,int crc2,word *fp,int mode)
{
  bool found=FALSE;
  int crc1a, crc2a;

  SetFP(11,sub,0xffff);
  *fp=0xffff;
  while (!found)
    {
      if (!LibFileFindNext(&fb,&finf,0x00)) return 61; /* not found */
      if (!LibFileRead(&fb,&finf))
  return 39; /* should not happen */
      crc(FBB,2600,&crc1a,&crc2a,mode);
      found=(crc1==crc1a)&&(crc2=crc2a);
    }
  *fp=finf.fp;
  return loadqmfp(addr,size,sub,*fp);
}

int loadqm(int addr,int size,int sub,int nr)
{
  word fp;
  fp=get_file_fp(11,sub,nr);
  return loadqmfp(addr,size,sub,fp);
}

void icon2fb(byte FAR *iconbuf)
{
  int xi=*(word FAR*)iconbuf;
  int xbi=(xi+7)/8; /* number of bytes per line */
  int yi=*(word FAR*)(iconbuf+2);
  int xb,yb;

  byte FAR * bptr=iconbuf+4;

  if (yi>130) yi=130; /* no more then 130 line (quickmemo limit) */

  fbbinit();
  for (yb=0;yb<yi;yb++)
    {
      for (xb=0;xb<xbi;xb++)
  {
    fbbputbyte(*(bptr++));
  }
      for (;xb<20;xb++)
  {
    fbbputbyte(0);
  }
    }

  for (yb=yi;yb<130;yb++)
    {
      for (xb=0;xb<20;xb++)
  {
    fbbputbyte(0);
  }
    }
}

int saveqmfp(int addr,int size,int sub,word *fp)
{
  char datestring[11];
  word year;
  byte month,day,hour,min,sec;
  int i;

  if (! checkicon(addr,size)) return 59;

#ifdef LSI_C
  icon2fb((byte FAR *)&(xdata[addr]));
#endif
#ifdef __HITACHI__
  toiconbuffer(addr,iconbuffer);
  icon2fb(iconbuffer);
#endif

  LibGetDateTime2(&year,&month,&day,&hour,&min,&sec);
  sprintf(datestring,"%4d/%2d/%2d",year,month,day);

  for (i=0;i<10;i++) fbbputbyte(0);

  /*  ! write date  */
  for (i=0;i<10;i++) { fbbputbyte(datestring[i]); }

  if (filewrite(11,sub,*fp)==0xffff) return 39;
  return 0;
}

int saveqm(int addr,int size,int sub,int nr)
{
  word fp;
  fp=get_file_fp(11,sub,nr);
  return saveqmfp(addr,size,sub,&fp);
}

int iconhash(int addr,int size,int *crc1,int *crc2,int mode)
{
  if (! checkicon(addr,size)) return 59;

#ifdef LSI_C
  icon2fb((byte FAR *)&(xdata[addr]));
#endif
#ifdef __HITACHI__
  toiconbuffer(addr,iconbuffer);
  icon2fb(iconbuffer);
#endif
  crc(FBB,2600,crc1,crc2,mode);
  return 0;
}

/* load / save settings */
void loadsettings(void)
{
  sfLoadRecord((uchar*)"OWBasic,prefs",sizeof(Pref),(byte*)&pref);
  if ((pref.ver)!=SETTINGSVER)
  {
    LibPutMsgDlg3((byte*)"Wrong settings version\rUsing defaults");
    default_prefs();
    savesettings();
    return;
  }
  show_compiling = pref.showcompiling;
  notshowinit = show_compiling;
  if (!show_compiling)
  {
    pref.showcompiling = TRUE;
    savesettings ();
  }
}

void savesettings(void)
{
  pref.ver = SETTINGSVER;
  sfSaveRecord((uchar*)"OWBasic,prefs",sizeof(Pref),(byte*)&pref);
}

int namedfile(const uchar *name,int *imode,int *isubmode)
{
  byte submode, mode;
  /* find or create named file */
  if (! LibSubEntrySave((byte*)name,&submode)) return 39;

  /* get mode/submode */
  LibGetAllEntry((byte*)name,&mode,&submode);
  *imode=mode >> 4; /* convert to OWB style */
  *isubmode=submode;
  return 0;
}

#define SIGLEN 60
/* Shared file access */
void mksig(uchar *sig,const uchar *pname,const uchar *sub,int n)
{
  const uchar *p1;
  uchar *p2;
  p2=sig;
  p1=pname;
  while ((*p1!=0)&&(p2-sig<n)) *(p2++)=*(p1++);
  if (strlen((char*)sub)>0)
    {
      if (p2-sig<n) *(p2++)=',';
      p1=sub;
      while ((*p1!=0)&&(p2-sig<n)) *(p2++)=*(p1++);
    }
  if (p2-sig<n) *(p2++)=0;
}

bool loadshared(const uchar *sig,const uchar *pname,uchar FAR *buf,int n)
{
  /* load record from shared file */
  uchar fsig[SIGLEN];
  mksig(fsig,pname,sig,SIGLEN);
  return sfLoadRecord(fsig,n,buf);
}

bool saveshared(const uchar *sig,const uchar *pname,uchar FAR *buf,int n)
{
  /* save record to shared file */
  uchar fsig[SIGLEN];
  mksig(fsig,pname,sig,SIGLEN);
  return sfSaveRecord(fsig,n,buf);
}

bool removeshared(const uchar *sig,const uchar *pname)
{
  /* remove record from shared file */
  uchar fsig[SIGLEN];
  mksig(fsig,pname,sig,SIGLEN);
  return sfRemoveRecord(fsig);
}

bool findshared(const uchar *sig,const uchar *pname,
    uchar *rname,int namelen,word *fp)
{
  uchar fsig[SIGLEN];
  if (strlen((char*)pname)==0)
    return sfFindRecord(sig,rname,namelen,fp);

  mksig(fsig,pname,sig,SIGLEN);
  return sfFindRecord(fsig,rname,namelen,fp);
}
/* -------------------------------------------- */
uchar recordname[]=PROGNAME",Overlay000";

void OverlayName(int i)
{
  recordname[17]='0'+(i % 10);
  recordname[16]='0'+(i / 10)%10;
  recordname[15]='0'+(i / 100)%10;
}

bool LoadOverlay(codet *addr,int n,int procnr)
{
  OverlayName(procnr);
  return sfLoadRecord(recordname,n,(byte*)addr);/* no limit */
}

bool SaveOverlay(codet *addr,int n,int procnr)
{
  OverlayName(procnr);
  return sfSaveRecord(recordname,n,(byte*)addr);
}

void DeleteOverlay(int nr)
{
  OverlayName(nr);
  sfRemoveRecord(recordname);
}
