#include <string.h>
#include "owbasic.h"
#include "graphic.h"

/* height of fonts */
#define FONTS 6
uchar fontheight[FONTS]={8,8,7,12,7,5};
/* gap between two textlines */
uchar fontgap[FONTS]={1,1,1,2,1,1};
#ifdef __HITACHI__
uchar libfonts[4]={IB_PFONT1,IB_PFONT2,IB_CG57FONT,IB_PFONT3};
#endif
int extstringsize(int font,unsigned char *text)
{
  int i, l;
  int ml;
  int sl;
  unsigned char a;
  l = strlen ((char*)text);
  sl = 0; ml = 0;
  for (i = 0; i < l; ++i)
  {
    a = text[i];
    if (a != NEWLINE)
    {
      if (font == 4)
        sl += fn57[a << 3 + 7];
      else sl += 4;
    }
    else
    {
      if (sl > ml)
        ml = sl;
      sl = 0;
    }
  }
  if (sl > ml)
    ml = sl;
  return (ml);
}

int stringsize (uchar *text,int font)
{
  unsigned char *hptr;
  unsigned char *hptr1;
  unsigned char hc;
  unsigned int ml,sl;
  if (font<4)
    {
      ml=0;
      hptr=text;
      hptr1=text;
      while (*hptr!=0)
  {
    while ((*hptr!=0)&&(*hptr!=NEWLINE)) hptr++;
    hc=*hptr; *hptr=0;
    sl=LibGetProStrSize(LFONT(font),hptr1);
    *hptr=hc;
    if (hc!=0) hptr++;
    hptr1=hptr;
    if (sl>ml) ml=sl;
  }
      return ml;
    }
  else
    return extstringsize(font,text);
}

int stringheight(uchar *text,int font)
{
  int c;
  c=countc(NEWLINE,text);
  return fontheight[font]*(c+1)+c*fontgap[font];
}

void extdrawstring (int font, int x0, int y0, unsigned char *text, int xl)
{
  int i, l;
  unsigned char bm, pattern;
  int xp, yp;
  int x, y, xe;
  unsigned char a;
  l = strlen ((char*) text);
  x = x0; y = y0;
  for (i = 0; i < l; ++i)
  {
    a = text[i];
    xe = x + fn57[a * 8 + 7];
    if (xe <= x0 + xl)
      for (yp = 0; yp < 7; ++yp)
      {
        pattern = fn57[a * 8 + yp];
        bm = 0x20;
        for (xp = x; xp < xe; ++xp)
        {
          if ((pattern & bm) > 0)
            LibGdsDotOn (xp, yp + y);
          else
            LibGdsDotOff (xp, yp + y);
          bm >>= 1;
        }
      }
    x = xe;
  }
}

int drawstring(uchar *text,int x,int y,int font,int xl)
{
  unsigned char *hptr;
  unsigned char *hptr1;
  unsigned char hc;

  if (xl < 0)
  {
    xl = x;
    x -= stringsize (text, font);
    if (x < 0)
      x = 0;
  }
  hptr=text;
  while (*hptr!=0)
  {
    if (!checkxy (x,y))
      return (65);
    hptr1 = hptr;
    while ((*hptr != 0) && (*hptr != NEWLINE))
      ++hptr;
    hc = *hptr;
    if (hptr != hptr1)
    {
      *hptr = 0;
      switch (font)
      {
      case 0:
      case 1:
      case 2:
      case 3:
        LibPutProStr (LFONT(font), x, y, hptr1, xl);
        break;
      case 4:
        extdrawstring (font, x, y, hptr1, xl);
        break;
      case 5:
        LibPut35Str (x, y, hptr1);
        break;
      }
      *hptr=hc;
    }
    if (hc!=0) hptr++;
    y+=fontheight[font]+fontgap[font];
  }
  return 0;
}

int drawtext(uchar *text, int x1,int y1,int x2,int y2, int font)
{
  int length,height;
  int xt,yt;
  int sx, sy;
  int fi;
  int ff[]={3,0,2,4};

  if (!checkxy2(x1,y1,x2,y2)) return 65;
  sy=y2-y1-5;
  sx=x2-x1-3;

  if (font==-1)
    {
      /* which font */
      fi=0;
      while ((fi<2)&&(stringheight(text,ff[fi])>sy)) fi++; /* biggest font */
      while ((fi<2)&&(stringsize(text,ff[fi])>sx)) fi++; /* biggest font */
      font=ff[fi];
    }

  if (font<0 || font>FONTS-1) return 74 ;
  length=stringsize(text,font);
  height=stringheight(text,font);
  xt=max((x1+x2+1-length)/2,x1);
  yt=max((y1+y2-height)/2+1,y1);
  return drawstring(text,xt,yt,font,x2-x1);
}

int textbox(uchar *text,int x1,int y1,int x2,int y2,int font)
{
  int rc;
  rc=Box(x1,y1,x2,y2,10);  /* clear box */
  if (rc!=0) return rc;
  rc=drawtext(text,x1+2, y1+2, x2-2, y2-2, font);
  if (rc!=0) return rc;
  return Box(x1,y1,x2,y2,1);   /* draw frame */
}

void floatinput (FLOATT* f)
{
  TCHSTS tsts;
  CALWRAM calram;
  int i;
  char nmcbuf[33];
  char* cp;
  FLOATT res;


  /* Touch settings */
  LibTchStackClr ();
  LibTchStackPush (TchHardIcon);
  LibTchStackPush (TchModeIcon);
  LibTchInit ();

  /* Initializing calculator. */
  LibCalKeyInit (&calram); /* Initializes calculator keyboard. */

  /* initialize CALRAM (MB,4.40) */
  sprintf (nmcbuf, FFORMATSTD, *f);
  /*LibPutMsgDlg (nmcbuf);*/
  cp = nmcbuf + strlen (nmcbuf) - 1;
  calram.calxbuf[0] = 0xFF;
  calram.calxbuf[1] = (*nmcbuf == '-') ? 0x0C : 0x04;
  for (i = 2; i < CAL_BUFSIZE; ++i)
    if (cp >= nmcbuf)
    {
      if (*cp == '.')
      {
        calram.calxbuf[0] = CAL_BUFSIZE - 1 - i;
        --cp;
      }
      if (*cp != '-')
        calram.calxbuf[i] = *cp;
      --cp;
    }
  if (calram.calxbuf[0] == 0xFF)
    calram.calxbuf[0] = CAL_BUFSIZE - 1 - i;

  LibCalKeyDsp (&calram); /* Displays Calculator keyboard. */
  LibPutDisp (); /* Reflects to the screen. */
  do
    LibCalKeyTchWait (&calram, &tsts); /* Waits for touch of calculator */
  while ((tsts.obj != OBJ_CAL_NEXT)
        && (tsts.obj != OBJ_CAL_EQUAL)
        && (tsts.obj != OBJ_HIC_ESC));

  res = 0;
  for (i = 2; i < 14; ++i)
    res = res * 0.1 + (calram.calxbuf[i] & 0x0F);
  if (calram.calxbuf[1] == 0x0C)
    res = -res;
  for (i = 0; i < calram.calxbuf[0]; ++i)
    res *= 10.0;
  if (tsts.obj != OBJ_HIC_ESC)
    *f = res;

}

void strinpbkg (const uchar* topic)
{
  LibClrDisp ();
  LibGdsBox (0, 0, 159, 85);
  LibDotOff (0, 0);
  LibDotOff (159, 0);
  LibDotOff (0, 85);
  LibDotOff (159, 85);
  drawstring ((uchar*) topic, 2, 2, IB_PFONT2, 157);
}

