#include <owbasic.h>
#include "inter.h"
#include <graphic.h>

/*
 * ICONS are the bitmaps of pv
 *
 * to stay compatible to pv<1600 the pv1600 stores icons
 * in the same way as array of int.
 * Because of changed sizes and endiness this requires
 * reorganisation before calling libfunctions which
 * take icons as byte[];
 */

const int icon_bitmask[16]={0x0080,0x0040,0x0020,0x0010,
          0x0008,0x0004,0x0002,0x0001,
          0x8000,0x4000,0x2000,0x1000,
          0x0800,0x0400,0x0200,0x0100};

#ifdef __HITACHI__
const int drawiconmodes[] = {IB_GPOVER, IB_GPOR, IB_GPAND, IB_GPOVER, IB_GPOVER}; /* IB_GPREV and IB_GPMESH are not available any more on PV-S1600 */
#else
const int drawiconmodes[] = {IB_GPOVER, IB_GPOR, IB_GPAND, IB_GPREV, IB_GPMESH};
#endif

void iconaddr0(int FAR *aa,int ox,int oy,int *bm,int FAR **p)
{
  int sx, sxm;
  int bo;
  sx=*aa;   /* read iconsize */
  sxm=((sx-1) | 7 ) +1; /* next multiple of 8 */
  bo=oy*sxm+ox;  /* bitoffset */
  *p=aa+2+(bo / 16);
  *bm=icon_bitmask[bo % 16];
}

bool iconaddr(int FAR *aa,int ox,int oy,int *bm,int FAR **p)
{
  int sx,sy;
  sx=*aa; sy=*(aa+1);   /* read iconsizes */
  if ((ox<0)||(ox>=sx)||(oy<0)||(oy>=sy)) return FALSE;
  iconaddr0(aa,ox,oy,bm,p);
  return TRUE;
}

/* Functions on OWB-Arrays */

#ifdef __HITACHI__
static int dummy; /* make iconbuffer word aligned */
byte iconbuffer[3204];

void toiconbuffer(int addr,byte *dest)
{
  short *sptr;
  byte *bptr;
  int sindex;

  sptr=(short*)dest;
  sindex=addr;
  *(sptr++)=xdata[sindex++]; /* x_size = width */
  *(sptr++)=xdata[sindex++]; /* y_size = height */
  bptr=(byte*)sptr;
  while (bptr<dest+3204)
    {
      *(bptr++)=xdata[sindex] & 0xff;
      *(bptr++)=(xdata[sindex] >> 8) & 0xff;
      sindex++;
    }
}

void fromiconbuffer(byte *src,int addr)
{
  byte *bptr;
  short *sptr;
  int sx,sy,sxm,rsize;
  int sindex=addr;

  sptr=(short*)src;
  sx=xdata[sindex++]=*(sptr++); /* x size */
  sy=xdata[sindex++]=*(sptr++); /* y size */

  sxm=(sx+7)/8; /* line size in bytes */
  rsize=(sxm*sy+1)/2+2; /* size in "ints" */

  bptr=(byte*)sptr;

  while (sindex<addr+rsize)
    {
      xdata[sindex++]=*(bptr+1)*256+*bptr;
      bptr+=2;
    }
}
#endif

bool checkicon(int aa,int size)
{
  int sx,sy; /* iconsize (pixel)*/
  int sxm;
  int rsize; /* required size (int) */
  sx=xdata[aa]; sy=xdata[aa+1];
  if (!checkxy(sx-1,sy-1)) return FALSE;/* invalid sizes ! */
  sxm=(sx+7)/8; /* line size in bytes */
  rsize=(sxm*sy+sizeof(int)-1)/sizeof(int)+2; /* size in ints */
  if (size<rsize) return FALSE;
  return TRUE;
}

int drawicon(int addr,int x,int y, int mode)
{
  int size;
#ifdef __HITACHI__
  byte *bptr;
  short *sptr;
  int sindex;
#endif
  size=xdata[addr-1];
  if (! checkicon(addr,size)) return 59;
  if (! checkxy2(x,y,x+xdata[addr]-1,y+xdata[addr+1]-1)) return 65;

#ifdef __HITACHI__
  toiconbuffer(addr,iconbuffer);
#endif
  if (mode == 0)
  {
#ifdef LSI_C
    LibPutGraph(x,y,(byte FAR *)&(xdata[addr]));
#endif
#ifdef __HITACHI__
    LibPutGraph(x,y,iconbuffer);
#endif
  }
  else
  {
    if ((mode < 0) || (mode > 4))
      return (38);
#ifdef LSI_C
    LibPutGraphO(x,y,(byte FAR *)&(xdata[addr]), drawiconmodes[mode]);
#endif
#ifdef __HITACHI__
    LibPutGraphO(x,y,iconbuffer, drawiconmodes[mode]);
#endif
  }
  return 0;
}

int geticon(int addr,int x1,int y1,int x2,int y2)
{
  int size;
#ifdef __HITACHI__
  byte *bptr;
  short *sptr;
  int sindex;
#endif
  if (!checkxy2(x1,y1,x2,y2)) return 65;
  size=xdata[addr-1];
  xdata[addr]=x2-x1+1;
  xdata[addr+1]=y2-y1+1;
  if (! checkicon(addr,size)) return 59;
  if (x2-x1+1<9) return 39;
#ifdef LSI_C
  LibGetGraph(x1, y1, x2-x1+1, y2-y1+1,(byte FAR*)&xdata[addr+2]);
#endif
#ifdef __HITACHI__
  ((short*)iconbuffer)[0]=x2-x1+1;
  ((short*)iconbuffer)[1]=y2-y1+1;
  LibGetGraph(x1, y1, x2-x1+1, y2-y1+1,&(iconbuffer[4]));
  fromiconbuffer(iconbuffer,addr);
#endif
  return 0;
}
