/*
 * TermLib - Library for Pocketviewer for terminal emulation
 *
 * Copyright (C) 2002 Wolfgang Ortmann
 * Contact: noo@uni-jena.de
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* input.c */
#include "stdio.h"
#include "define.h"
#include "libc.h"
#include "trm.h"

#if FALSE
int InputTrm(uchar *ib,int cnr)
{
  uchar *hp;
  byte aline, acol;
  int ready;
  int cnt;
  TCHSTS tsts;
  byte kycd;
  ModeTrm(M_SHORT);
  LibKeyInit();
  LibDispKey();
  ready=FALSE;
  spline=cursorline;
  spcol=cursorcol;
  while (!ready)
    {
      DisplayTrm();
      kycd=LibGetKeyM(&tsts); /* get key or other events */
      if (kycd!=KEY_NONE)     /* any Key */
  {
    if ((kycd==0xF0)||(kycd==0xF7)) ready=TRUE;
    else
      if (kycd!=0xf8) OutTrm(kycd);
  }
      else
  {                     /* no key */
    switch (tsts.obj)
      {
      case OBJ_HIC_ESC:
        ready=TRUE;     /* ESC pressed */
        break;
      } /* switch (tsts.obj) */
  } /* if (kycd!=KEY_NONE) */
    } /* while (!ready) */

  acol=cursorcol; cursorcol=spcol;
  aline=cursorline; cursorline=spline; /* start position */
  cnt=0;
  hp=ib;
  while ( (aline>cursorline)||
       ((aline==cursorline)&&(acol>cursorcol))
     )
  {
  if (cnt<cnr) *(hp++)=trmbuffer[cursorline*TRMCOLS+cursorcol];
  cnt++;
  cursornext();
     }

  *(hp++)=0;
  OutTrm(0xF0);
  ModeTrm(M_LONG);
  DisplayTrm(); /* update display */
  switch (kycd)
    {
    case 0xF0: return TRM_ENTER;
    case 0xF7: return TRM_NEXT;
    default: return TRM_ESC;
    }
}
#endif


int InputTrm (uchar* ib, int cnr)
{
  int retval;

  ModeTrm (M_SHORT);
  DisplayTrm ();
  ib[0] = '\0';
  retval = inputstr ((char*) ib, 2, 12, 157, 8, IB_CG57FONT, cnr);
  ModeTrm (M_LONG);
  OutputTrm (ib);
  OutTrm (0xF0); /* newline */
  DisplayTrm ();
  return (retval);
}


int inputstr (char* str, int x1, int y1, int x2, int lnr, byte font, int max)
{
  TXTP txtp;
  TCHSTS tsts;
  TCHTBL ttbl[3];
  T_SCR_POS tscr;
  byte kycd;
  word gdt[] = {0xFFFF, 0};


  txtp.st_x = x1; /* Start coordinate (X) of text display */
  txtp.st_y = y1; /* Start coordinate (Y) of text display */
  txtp.ed_x = x2; /* End coordinate (X) of text display */
  txtp.it_y = 9; /* Text display line spacing (Y) */
  if (font == IB_PFONT3)
    txtp.it_y += 4;
  txtp.MAXGYO = lnr; /* Number of text display lines */
  txtp.font = font; /* Display font type */
  txtp.csen = TRUE; /* Cursor display disabled */
  txtp.rtnen = 0x02; /* CR code display disabled */
  txtp.maxmj = max; /* Maximum number of input characters */
  txtp.txbf = (byte*) str; /* Text buffer address specification */
  txtp.gdcmt = gdt; /* Guidance comment table */
  txtp.txtobj	= OBJ_TXTTCH;		/* Object when text area is touched.  */
  txtp.sbrobj	= OBJ_SCR_BAR;				/* Object when the scroll bar is touched. */
  txtp.tchtb	= ttbl;		/* Pointer for text scroll bar area  */
  txtp.s_pos	= &tscr;			/* Pointer for text and scroll bar information */

  LibTchStackClr ();
  LibTchStackPush (NULL);
  LibTchStackPush (TchHardIcon);
  /*LibTchStackPush (TchModeIcon);*/
  LibTchInit ();

  LibTxtInit (&txtp); /* Initialization of text input */
  LibTxtTchSet (&txtp);

  while (TRUE)
  {
    if (LibTxtDsp (&txtp))
      LibPutDisp ();
    kycd = LibGetKeyM (&tsts); /* get key or other events */
    if (kycd != KEY_NONE)     /* any Key */
    {
      if (kycd == 0xF0)
        return (TRM_ENTER);
      if (kycd == 0xF7)
        return (TRM_NEXT);
    }
    else /* no key */
      if (tsts.obj == OBJ_HIC_ESC) /* ESC pressed */
        return (TRM_ESC);

    LibTxtInp (kycd, &tsts, &txtp);
  }
}

