#include <string.h>
#include "owbasic.h"
#include "comp.h"
/*
   Compiler list management

   The list of symbols has the following structure:

   Each entry contains
   - xdata[n]
       high byte = type
       low byte = entry length len
   - xdata[n+1] runtime address
   - xdata[n+2] element number total
   - xdata[n+3] element number per line
   - xdata[n+4] element size
   - xdata[n+5] ...
       identifier (character string, finished with '\0')
   - xdata[n+len-1] n -> pointer to start of entry

   Constants managment:
     constant values for integers and floats
*/

int listnext; /* next entry (compile time) */

/* int datanext; next useable address (for runtime), now defined in bin.c */
/* int firstconst; first address used for constants, now defined in bin.c */

int lastaddr; /* address of the last created entry */

void tlsplit(int tl,uchar *typ,uchar *len)
{
  *typ=(tl >> 8) & 0xff;
  *len= tl & 0xff;
}

int newentry(int dtype,const uchar *name)
{
  /* warning: length of name is max(IDENTLEN,NAMELEN+1)*/
  int adr,namelength;
  int datasize, headersize;
  uchar FAR *dptr;
  const uchar *sptr;
  if (listnext>firstconst-(NAMELEN+1+1)/sizeof(int)-11)  /* check free symbol space */
    serror(5);
  adr=listnext;                         /* position of this entry */
  sptr=name;                            /* address of identifier */
  dptr=(uchar FAR *)(&xdata[adr+5]);    /* destination address */
  while (*sptr!=0) *(dptr++)=*(sptr++); /* copy identifier */
  *(dptr++)=0;                          /* null as delimiter */
  namelength=strlen((char*)name)+1;     /* number of bytes for name */
  namelength=(namelength+sizeof(int)-1)/sizeof(int); /* string length in int */

  /* determine datasize (in sizeof(int))*/
  if (dtype & KREFERENCE)
    {
      datasize=1; headersize=0; /* pointer only, no element count */
    }
  else
    {
      switch(dtype & TYPEMASK)
  {
  case DFPTR:    /* the value itself + len values */
  case DINTEGER: /* the value itself + len values */
  case DBOOLEAN: /* the value itself + len values */
  case DCHAR:    /* the value itself + len values */
  case DSTRING:  /* a pointer to the string + len values */
    datasize=1; headersize=2;
    break;
  case DFLOAT:  /* FLOATT size + len values */
    datasize=FLOATSIZE; headersize=2;
    break;
  case DCONSTANT:  /* no runtime length, only at compiletime */
  case DLABEL:
  case DREFERENCE:
    datasize=0; headersize=0;
    break;
  }
    }

  /* allocate memory */
  if (dtype & KCONST)
    xdata[adr+1]=firstconst;     /* will be changed later */
  else
    {
      if ((dtype & TYPEMASK) == DLABEL) xdata[adr+1]=0; /* new label */
      else
  /* address of data at runtime
   * pointer points direct to data after header
   */
  {
    datanext+=headersize; /* reserve header */
    xdata[adr+1]=datanext; /* store runtime address of data */
    datanext+=datasize;  /* reserve data space */
  }
    }

  if (datanext>=firstconst) serror(33); /* not enough space */
  xdata[adr+2]=1; /* number of elements (total) */
  xdata[adr+3]=1; /* number of elements per line */
  xdata[adr+4]=datasize;                /* size of *one* element */
  xdata[adr]=(dtype << 8)+(namelength+6); /* store type and length of entry */
  xdata[adr+5+namelength]=adr;          /* last value points to start of entry */
  listnext+=namelength+6;              /* place of next entry */
  xdata[listnext]=0;                   /* end of list */
  lastaddr=adr+1;                      /* address of last entry */
  return lastaddr;
}

int expandentry(int addr,int el,int el_per_line)
   /* expand size of runtime memory (arrays) */
{
  if (addr!=lastaddr) serror(31);  /* only possible for last entry */
  xdata[addr+1]=el;                /* number of elements of array */
  xdata[addr+2]=el_per_line;       /* number of elements per row */
  datanext+=(el-1)*xdata[addr+3];  /* allocate memory */
  if (datanext>=firstconst) serror(33); /* not enough space */
  return 0;
}

int searchentry_token(uchar *dtype)
{
  uchar n[NAMELEN+1];
  tokncpy(n,NAMELEN+1);
  searchentry_name(dtype,n);
}

int searchentry_name(uchar *dtype,const uchar *name)
{
  uchar n[NAMELEN+1];
  int ptr;
  uchar varlen,vartype;
  int var;
  uchar *p1;
  uchar FAR *pf;
  uchar rtype;
  rtype=*dtype & TYPEMASK;
  ptr=0; /* start search at beginning */
  var=-1; /* not found */
  /* copy and normalize name */
  strcpy_name(n,name);
  strlower(n); /* normalize name (lower case) */
  while ((xdata[ptr]!=0)&&(var<0)) /* search entry */
    {
      tlsplit(xdata[ptr],&vartype,&varlen); /* data type and length of entry */

      p1=n; pf=(uchar FAR*)&(xdata[ptr+5]);
      while ((*p1!=0)&&(*p1==*pf)) {pf++; p1++;} /* compare names */
      if (*p1==*pf) /* name found */
  {
          if ((rtype==DUNKNOWN)|| /* search type unknown -> OK */
        ((vartype & TYPEMASK)==rtype)) /* check given type */
      {
        /* found !*/
        *dtype=vartype; /* found type */
        var=ptr+1;  /* location of entry */
      }
  }
      ptr+=varlen; /* next entry */
    }
  return var;
}

int findentry_token(uchar *dtype)
{
  uchar n[NAMELEN+1];
  tokncpy(n,NAMELEN+1);
  findentry_name(dtype,n);
}

int findentry_name(uchar *dtype, const uchar *name)
{
  uchar n[NAMELEN+1];
  int var;

  var=searchentry_name(dtype,name);
  if (var<0) /* if not found */
  {
    if (declaration_required && (*dtype != DLABEL))
      serror (77); /* undefined variable */
    /* copy and normalize name */
    strcpy_name(n,name);
    strlower(n); /* normalize name (lower case) */

    *dtype=DT(*dtype);
    var=newentry(*dtype,n);
  }
  return var;
}

#if FALSE
void listshow()
{
  int ptr;
  uchar FAR *hptr;
  uchar t,elength,l;
  for (l=0; l<15; l++) OutTrm('\r');
  ptr=0;
  while (xdata[ptr]!=0)
    {
      tlsplit(xdata[ptr],&t,&elength);
      hptr=(uchar FAR*)&(xdata[ptr+5]);
      while (*hptr!=0) OutTrm(*(hptr++));
      OutTrm(' ');
      OutputITrm(t);
      OutputITrm(xdata[ptr+1]);
      OutputITrm(xdata[ptr+2]);
      OutTrm('\r');
      ptr=ptr+elength;
    }
}
#endif

void listreset()
{
  listnext=0;
  datanext=0;
  xdata[0]=0;
  firstconst=DATASIZE;
}

void listhide(int pos)
{
  /* removes local entries in symboltable logically
     labels and references are removed
     the name of variables is reduced to an emptystring and the
     entry is compacted */
  int sptr=pos; /* from .. */
  int dptr=pos; /* to .. */
  uchar typ, len;
  check_labels(); /* link labels */
  while (xdata[sptr]!=0)
    {
      tlsplit(xdata[sptr],&typ,&len);
      if ((typ==DLABEL)||(typ==DREFERENCE)||(typ==DCONSTANT))
  {
    sptr+=len; /* ignore (delete) entry*/
  }
      else
  {
    xdata[dptr+1]=xdata[sptr+1]; /* copy entry to new position */
    xdata[dptr+2]=xdata[sptr+2];
    xdata[dptr+3]=xdata[sptr+3];
    xdata[dptr+4]=xdata[sptr+4];
    xdata[dptr+5]=0; /* no name */
    xdata[dptr+6]=dptr;
    xdata[dptr]=(typ<<8)+7;
    dptr+=7;
    sptr+=len;
  }
      listnext=dptr; /* set pointer for next free place */
    }
  xdata[dptr]=0; /* mark end */
}

void check_labels()
{
  int ptr;
  uchar t, l;
  int maddr;
  uchar label[80];
  PC baddr, raddr;

  /*  listshow(); */

  ptr=0;
  while (xdata[ptr]!=0)
    {
      tlsplit(xdata[ptr],&t,&l);
      if ((t & TYPEMASK)==DREFERENCE)
  {
    maddr=xdata[ptr+1];/* pointer to label data */
    baddr.p=(codet**)xdata[ptr+2];/* where to put address */
    raddr.c=(codet*)xdata[maddr];/* labeladdress */
    if (raddr.c==NULL) /* Label undefined */
      {
        /* label name at xdata[ptr+4] */
        strcpy_nf(label,(uchar FAR*)&(xdata[maddr+4]));
        serror1(6054,label);
      }
    if (*baddr.p!=0)
      {
        LINKJMP(baddr.i,raddr.c);
      }
    else
      *baddr.p=raddr.c;
  }
      ptr+=l;
    }
}

void create_variables ()
{
  int aentry;       /* pointer for entries */
  int ptr;          /* pointer in temporary list */
  uchar typ, len;   /* typ of variables and len of entry */
  int addr;         /* runtime address of variables */
  int length;       /* number of elements */
  int row_length;   /* number of elements per line */
  int i;            /* counter */

  aentry=listnext;  /* set aentry after last entry */
  ptr=firstconst-4; /* start temporary table before constants */
  while (aentry>0)
    {
      aentry=xdata[aentry-1];           /* previous entry */
      if (ptr<listnext) serror(5);
      tlsplit(xdata[aentry],&typ,&len); /* type of variable (and len)*/
      switch (typ & TYPEMASK)
  {
  case DBOOLEAN:
  case DINTEGER:
  case DCHAR:
  case DFLOAT:
  case DSTRING:
  case DFPTR:
    xdata[ptr]=typ;
    xdata[ptr+1]=xdata[aentry+1]; /* runtime address */
    xdata[ptr+2]=xdata[aentry+2]; /* number of elements */
    xdata[ptr+3]=xdata[aentry+3]; /* number of elements per row */
    ptr-=4;
  default:            /* ignore labels, constants and references*/
    break;
  }
    }
  ptr+=4;                   /* back to first entry */

  while (ptr<firstconst)
    /* create variables = size + data */
    {
      /* read entry */
      typ=xdata[ptr]; /* type */
      addr=xdata[ptr+1]; /* runtime addr of variable */
      length=xdata[ptr+2]; /* number of elements (total) */
      row_length=xdata[ptr+3]; /* number of elements (per line) */
      if (!(typ & KCONST))
      {
        xdata[addr]=0;
        if (!(typ & KREFERENCE))
        {
          xdata[addr-2]=row_length; /* elements in array row */
          xdata[addr-1]=length;     /* number of elements in array */

          for (i = 0; i < length; ++i)
          {
            /*switch(typ & TYPEMASK)
            {
            case DBOOLEAN: xdata[addr+i]=FALSE; break;
            case DSTRING: xdata[addr+i]=0; break;
            case DCHAR:
            case DFPTR:
            case DINTEGER: xdata[addr+i]=0; break;
            case DFLOAT: *(FLOATT FAR*)(&xdata[addr+FLOATSIZE*i])=0.0;
              break;
            }*/
            if ((typ & TYPEMASK) == DFLOAT)
              *(FLOATT FAR*) (&xdata[addr + FLOATSIZE * i]) = 0.0;
            else
              xdata[addr + i] = 0;
          }
        }
      }
      ptr += 4;
    }
}

/*
   -------------  constants management ------------------
*/
void iconst(int i)
{
  firstconst--; /* reserve space */
  if ((firstconst<listnext)|| /* enough memory ? */
      (firstconst<datanext)) serror(33);
  xdata[firstconst]=i; /* store value */
}

void fconst(FLOATT f)
{
  firstconst-=FLOATSIZE; /* reserve necessary space */
  if ((firstconst<listnext)|| /* enough memory ?*/
      (firstconst<datanext)) serror(33);
  *((FLOATT FAR *)( &(xdata[firstconst]) ))=f; /* store value */
}
