#include "modeldef.h"
#include <shared.h>
#include "owbasic.h"
#include "comp.h"
#include "rc.h"
#include "graphic.h"
#include "modecall.h"
#include "inter.h"
#include "file.h"

static void NEAR welcome(void);

Pref pref;
Switches switches;

PC pc; /* code pointer for compiler and interpreter */
PC lastpc;

static int isAutorun;

bool isFindRTE;
bool overlays_used = FALSE;
byte declaration_required = 0;

/* array for "big data" - symbols (compile time), user variables (run time) */
int FAR xdata[DATASIZE];

#if FALSE
/* pointers to xdata - array for user variables */
int tchxi=0; int tchyi=1;
int tchobji=2; int tchstsi=3; /* values used before start of compiler */
int pvmodeli=4; int owbversioni=5;
int FAR *tchxp=&(xdata[0]);
int FAR *tchyp=&(xdata[1]);
int FAR *tchobjp=&(xdata[2]);
int FAR *tchstsp=&(xdata[3]);
int FAR *pvmodelp=&(xdata[4]);
int FAR *owbversionp=&(xdata[5]);
#endif

int error, inputrc;
/* binary code */
codet bin[PROG_SIZE];

/* the one and only filebuffer */
FILE_BUF fb;
/* name of program = name of main memo */
uchar progname[NAMELEN];

bool show_compiling = TRUE;
bool save_to_cache = FALSE;

int pvmodel;

#ifdef LSI_C
int getmodel (void);
#endif

/* how we were called */
word mymode;
word mystatus;

/* how we return */
byte oret;

void Restart(byte status)
{
  StopTimer();
  LibModeJump(mymode,status);
  LibJumpMenu(); /* if modejump fails - should never happen */
}

void default_prefs ()
{
  pref.progsub       = (byte) 5;
  pref.includesub    = (byte) 5;
  pref.statistics    = (byte) FALSE;
  pref.startmenu     = (byte) M_FILE;
  pref.finalwait     = (byte) 1;
  pref.askabort      = (byte) TRUE;
  pref.clearoverlay  = (byte) FALSE;
  pref.autorun       = (byte) FALSE;
  pref.restart       = (byte) FALSE;
  pref.rterror       = (byte) FALSE;
  pref.cacheing      = (byte) FALSE;
  pref.showcompiling = (byte) TRUE;
  pref.owbparam[0]   = (uchar) 0;
}

static void NEAR welcome()
{
  uchar versions_string[10];
  /* welcome screen */
  LibClrDisp();
  LibPutBoxSub(5,135,80,155,(byte)1);
  sprintf((char*)versions_string,"%d",current_version);
  LibStringDsp((byte*)PROGNAME,(word)10,(word)140,(word)125,LFONT ((byte)3));
  LibStringDsp((byte*)versions_string,(word)60,(word)146,(word)125, LFONT ((byte)1));
  LibPutDisp();
}

static void NEAR Select()
{
  int menu;
  menu=pref.startmenu;
  while (menu!=M_READY)
    {
      switch (menu)
  {
  case M_FILE: menu=filemenu(); break;
  case M_PROGRAM: menu=programmenu(); break;
  case M_PREFS:
    prefsmenu();
    menu=pref.startmenu;
    break;
  default:
    menu=M_FILE; /* repair wrong assignments */
    break;
  }
    }
}

char FAR cantload[]="Can't load program";
char FAR statmsg[]="Program: %d of %d\r\rSymbol: %d of %d\r\r"
"Data: %d of %d\r\rStart: %d\r\r"
"Procedures: %d of %d";


void ClearOverlays (void)
{
  sffindfile (&fb);
  recordname[15] = 0;
  finf.fp = 0xFFFF;

  while (LibFileFindNext (&fb,&finf,0x00))
  {
    LibFileReadEx (&fb, &finf, (3 * 1024) / FILE_BLOCK_SIZE);
    FBB[15] = 0;
    if (strcmp ((char*) (FBB), (char*) recordname) == 0)
    {
      LibFileRemove(&fb,&finf);
      finf.fp = 0xFFFF;
    }
  }
}

void main()
{
  int /*i,*/ j;
#ifndef SMALL_NO_BUILTIN
  int k;
#endif
  int submode;
#ifdef LSI_C
  word segm, offs;
#endif
#ifdef __HITACHI__
  void *seg;
#endif
  byte *hptr;

  oret=O_NORMAL; /* prepare normal return */

#ifdef LSI_C
  LibGetMode(&mymode,&mystatus,&segm,&offs); /* how were we called */
#endif
#ifdef __HITACHI__
  LibGetMode(&mymode,&mystatus,&seg);        /* how were we called */
#endif

#ifdef LSI_C
  pvmodel = getmodel ();
#endif
#ifdef __HITACHI__
  pvmodel = 5;
#endif


  if (mystatus==STATUS_NORMAL) {
    /* normal mode (called from menu) */
    welcome();  /* welcome screen */
  }

  if (mystatus == STATUS_CACHE)
    save_to_cache = TRUE;

#ifndef SMALL_NO_BUILTIN
  if (mystatus==STATUS_BUILTIN) {
    /* initialize program, variables and constants */
    for (i=0; i<initcodesize; i++) bin[i]=ibin[i];
    i=0; j=0;
    while (i<datanext)
      {
  /*xdata[i++]=xdata_var[j++];*/
  xdata[i++] = XDATA_VAR(j);
  ++j;
  for (k=0;k</*xdata_var[j]*/ XDATA_VAR(j);k++,i++) xdata[i]=0;
  j++;
      }
    for (j=0,i=firstconst; i<DATASIZE; i++,j++) xdata[i]=xdata_const[j];
  }
#endif

#ifdef LSI_C
  /* set int vector for division by zero */
  setintvecdiv0();
#endif

  InitTimer();
#ifdef __HITACHI__
  InitBreakTimer();
#endif
  /* compile time values for model and version */
  *owbversionp=OWBVersion;
#ifdef __HITACHI__
  *pvmodelp=1600;
#endif
#ifdef LSI_C
  *pvmodelp=450;
#endif

  /* load setting and initialise switches and variables */

  loadsettings();

  InitTrm();
  notshowinit = 0;

  switches.last=PROG_SIZE-10;
  switches.remote=(byte)FALSE;
  switches.calllevel=(byte)0;
  switches.statistics=pref.statistics;
  switches.cached = FALSE; /* current program cached? */
  switches.cacheing = pref.cacheing; /* cacheing allowed? */

  isAutorun=FALSE;
  isFindRTE=(mystatus==STATUS_FIND_RTE);

  IncStackPtr=IncStack;
  ITOS.min=0;
  ITOS.progtype=MMAIN;

  if (isFindRTE)
    {
      /* find runtime error */
      FBBNUM=3072;
      loadshared((byte*)"rterror",(byte*)PROGNAME,FBB,3072);
      fbbinit();
      fbbstring(progname);
      fbbint(&(switches.last));
      strcpy((char*)ITOS.fname,(char*)progname);
    }

  if (mystatus==STATUS_RC) switches.remote=(byte)TRUE; /* remote mode */

  if (mc_isCall ())
  {
    switches.calllevel = mc_CallLevel ();
    if ((mystatus & 0x0F) == 0x01) /* called from IEdit */
      switches.cacheing = FALSE; /* do not search for cached program */
  }

#ifndef SMALL_NO_BUILTIN
  if (mystatus==STATUS_BUILTIN) goto start;
#endif

  /* selection of program */

  prog=fb.fbuf.text.fsb_text_buf_;

#ifdef REMOTE_CONTROL
  if (switches.remote) /* are we in remote control mode? */
    {
      switches.statistics=FALSE;
      /* selection from pc */
      if (!rc_selprog())
  {
    /* verify abort */
    if (Ask((uchar*)"Abort?")) LibJumpMenu(); /* Aborted */
    Restart(STATUS_RC);
  }
    }
  else
#endif

    if (mc_isCall())
      {
  switches.statistics=FALSE;

  if (!mc_Init()) goto end;
  mc_ReadParam(&submode,ITOS.fname,  /* category and name */
         &j);              /* dummy parameters (cursor) */

  if (submode<0) ITOS.min=0;     /* not given, use default */
  else ITOS.min=submode;

  if (ITOS.fname[0]=='!' && ITOS.fname[1]==' ')
    {
      /* skip "! " prefix */
      strcpy((char*)ITOS.fname,(char*)&(ITOS.fname[2]));
    }
      }
    else
      {
  /* autorun or find runtime error or interactive selection mode */
  if (!isFindRTE)
    {
      if (pref.autorun)
        {
    isAutorun=TRUE;
    ITOS.min=pref.progsub;
    strcpy_name(ITOS.fname,(uchar*)"autorun");
    strcpy((char*)progname,(char*)ITOS.fname);
    if (!loadprog()) Select();
        }
      else
        Select();
    }
      }
  strcpy((char*)progname,(char*)ITOS.fname);
  if (!loadprog())
    {
#ifdef REMOTE_CONTROL
      if (switches.remote)
  {
    rc_finish(FALSE,(uchar*)Near(cantload));
    Restart(STATUS_RC);
  }
#endif
      LibPutMsgDlg((byte*)Near(cantload));
      goto end;
    }

  for (j = 0; j < NUM_PROC; ++j)
    proc_table[j].name[0] = '\0';

  /* compile process */
  if (compile(switches.last)!=0)
    {
      /* compiler error */
      oret=O_COMPILE_ERROR;
      goto end;
    }

#ifdef REMOTE_CONTROL
  if (SendBinary)
    {
      rc_send_binary();
    }
#endif

  if (switches.statistics) /* print statistics */
    {
      LibPutMsgDlg((byte*)Near(statmsg),
       pc.c-bin,PROG_SIZE,listnext,DATASIZE,datanext,DATASIZE,
       bin_start.c-bin,last_proc + 1,NUM_PROC);
    }

  /* prepare program execution */
#ifndef SMALL_NO_BUILTIN
 start:
#endif
  LibTchStackClr();
  LibTchStackPush(NULL);
  LibTchStackPush(TchAllDsp);
  LibTchStackPush(TchHardIcon);

#ifdef LSI_C
  InitTimer();
#endif
  /* execute the program */

  if (!Exec(bin_start)) oret=O_RUNTIME_ERROR;

 end:
  if (pref.clearoverlay && overlays_used)
  {
    /*for (i=0;i<NUM_PROC;i++)
    DeleteOverlay(i);*/
    ClearOverlays ();
  }

#ifdef REMOTE_CONTROL
  if (switches.remote)
    {
      if (oret==O_NORMAL) rc_finish(TRUE,(uchar*)"");
      Restart(STATUS_RC);
    }
#endif

  if (switches.calllevel>0)
    {
      /* return to caller */
      if (oret==O_COMPILE_ERROR)
  {
    /* prepare filename (memoname) of error */
    hptr=prog_start;
    while ((*hptr!=0x0d)&&(*hptr!=0)) hptr++;
    *hptr=0;

    mc_Init(); mc_Skip(); mc_Skip();
    mc_WriteParam(ITOS.min,prog_start,prog-prog_start);
  }
      StopTimer();
      LibModeJump(mc_CallerMode(),MC_RETURNVALUE+switches.calllevel-1);
    }

  /* finish OWBasic */
  if ((pref.restart)&&(!isAutorun))
    {
      Restart(STATUS_NORMAL); /* restart in "normal mode" */
    }

  /* normal call */
  if (oret==O_RUNTIME_ERROR)
    if (pref.rterror)
      Restart(STATUS_FIND_RTE);

  StopTimer();
  LibJumpMenu();
}
