#include "owbasic.h"
/* implementation of string storage */
int firststring;
int StringStackPointer;

uchar s1[MAXSTRLEN+1];
uchar s2[MAXSTRLEN+1];
uchar s3[MAXSTRLEN+1];
uchar s4[MAXSTRLEN+1];
uchar s5[MAXSTRLEN+1];
uchar *SStack[]={s1,s2,s3,s4,s5};
uchar *stos;
uchar *sst1;
uchar *sst2;

/* management of basic strings */
void resetstrings(void)
{
  firststring=firstconst;
}

int insertstring(int ref,const uchar *st)
{
  /*
    store char[] as Basic-String
    store "pointer" to string in xdata[ref]
    ! must be the only pointer to this string
  */
  int len = 0;
  int rlen;
  int addr;
  const uchar *ptr = st;
  uchar FAR *dptr;
  if (*st == 0)
  {
    /* null-string is not really stored */
    xdata[ref] = 0;
    return (0);
  }
  while ((*ptr != 0) && (len < MAXSTRLEN))
  {
    ++ptr;
    ++len;
  }
  if (len >= MAXSTRLEN)
    rterror (FN_STRING, 32);

  /*
    calculate number of integers needed to store string:
    - 2 integers (record length, pointer to reference)
    - string itself
    - 1 integer (backward link to begin)
  */
  rlen = ((len + 1 + sizeof (int) - 1) / sizeof (int)) + 3;
  addr = firststring - rlen;
  if (addr < datanext)
  {
    garbagecollection ();
    addr = firststring - rlen;
    if (addr < datanext)
      rterror (FN_STRING, 33);
  }
  dptr = (uchar FAR *) &(xdata[addr + 2]);
  ptr = st;
  while (*ptr != 0)
    *(dptr++) = *(ptr++);
  *dptr = 0;
  xdata[firststring - 1] = addr; /* backward link at end of record */
  xdata[addr + 1] = ref; /* pointer to pointer to string */
  xdata[addr] = firststring - addr; /* len of record */
  xdata[ref] = addr; /* pointer to string */
  firststring = addr; /* start of stringarea */
  return (addr);
}

#if FALSE
void deletestring(int addr)
{
  if (addr<=0) return; /* was nullstring, nothing to do */
  if (xdata[addr]<0) return; /* already deleted ? shouldn't happen! */
  xdata[addr]=-xdata[addr]; /* (neg. length) mark as deleted */
  xdata[xdata[addr+1]]=0; /* set reference to NULL */
  xdata[addr+1]=0; /* no reference exist */
  if (addr==firststring) firststring-=xdata[addr];
}
#else
void deletestring (int addr)
{
  int a2 = xdata[addr];

  if (a2 > firstconst) /* if stringconstant */
    a2 = 0; /* set pointer to 0 */
  if (a2 <= 0)
    return; /* was nullstring, nothing to do */
  if (xdata[a2] < 0)
    return; /* already deleted ? shouldn't happen! */
  xdata[a2] = -xdata[a2]; /* (neg. length) mark as deleted */
  xdata[xdata[a2 + 1]] = 0; /* set reference to NULL */
  xdata[a2 + 1] = 0; /* no reference exist */
  if (addr == firststring)
    firststring -= xdata[a2];
}
#endif

void garbagecollection(void)
{
  int hptr=firststring;
  int i, len;
  while (hptr<firstconst)
    {
      while ((hptr<firstconst)&&(xdata[hptr]>0)) hptr+=xdata[hptr];
      if (hptr<firstconst)
  {
    len=-xdata[hptr];
    for (i=hptr-1;i>=firststring;i--) xdata[i+len]=xdata[i];
    firststring+=len;
    hptr+=len;
  }
    }

  hptr=firststring;
  while (hptr<firstconst)
    {
      xdata[xdata[hptr+1]]=hptr;
      hptr+=xdata[hptr];
    }
}

/* --------------------------------------- */
void putvar(int vadr,const uchar *value)
{
  /* store *value in variable */
  /*LibPutMsgDlg ("%i, %i", xdata[vadr], xdata[xdata[vadr]]);*/
  /*deletestring(xdata[vadr]);*/
  deletestring (vadr);
  insertstring (vadr, value);
}

void getvar(int vadr,uchar *where)
{
  /* store string variable in char string */
  uchar *dptr=where;
  int saddr=xdata[vadr];
  uchar FAR *sptr=(uchar FAR *)&(xdata[saddr+2]);
  if (saddr!=0) /* if not empty (null) string */
    while (*sptr!=0) *(dptr++)=*(sptr++);
  *dptr=0;
}

/* string stack operations */
#define SP setpointers()

void setpointers(void)
{
  stos=SStack[StringStackPointer];
  sst1=SStack[StringStackPointer-1];
  sst2=SStack[StringStackPointer-2];
}

void spush(int vadr)
{
  spush1();
  getvar(vadr,STOS);
}

void spush1(void)
{
  StringStackPointer++;
  if (StringStackPointer>=S_STACK_SIZE) rterror(FN_STRING,35);
  SP;
}

void spop(int vadr)
{
  putvar(vadr,STOS);
  if (StringStackPointer<0) rterror(FN_STRING,34);
  StringStackPointer--;
  SP;
}

void spop1(void)
{
  if (StringStackPointer<0) rterror(FN_STRING,34);
  StringStackPointer--;
  SP;
}

void spop2(void)
{
  if (StringStackPointer<1) rterror(FN_STRING,34);
  StringStackPointer-=2;
  SP;
}
