#include <string.h>

#include "owbasic.h"
#include "ostring.h"

#undef tolower

/* tools for uchar[] */
int strcmp_nf(const uchar *sn,const uchar FAR *sf)
{
  /* compare near uchar[] and FAR uchar[]*/
  const uchar *np=sn;
  const uchar FAR *fp=sf;
  while ((*np!=0)&&(*np==*fp)) { np++; fp++; }
  if (*np>*fp) return 1;
  if (*np<*fp) return -1;
  return 0;
}

void strcpy_nf(uchar *sn,const uchar FAR *sf)
{
  /* copy FAR_string sf to near_string sn*/
  uchar *np=sn;
  const uchar FAR *fp=sf;
  while (*fp!=0) { *(np++)=*(fp++); }
  *np=0;
}

#ifndef NEVER_DEFINED
/* copy "special" strings (with given stringlength)*/
void strcpy_ident(uchar *d,const uchar *s)
{
  strncpy((char*)d,(char*)s,IDENTLEN);
}

void strcpy_string(uchar *d,const uchar *s)
{
  strncpy((char*)d,(char*)s,MAXSTRLEN);
}

void strcpy_name(uchar *d,const uchar *s)
{
  strncpy((char*)d,(char*)s,NAMELEN);
}
#endif

void strlower(uchar *s)
{
  /* convert uchar[] to lower case letters */
  while (*s)
  {
    *s = tolower (*s);
    ++s;
  }
}

/* string normalization */
void strupper(uchar *s)
{
  /* convert uchar[] to upper case letters */
  while (*s)
  {
    *s = toupper (*s);
    ++s;
  }
}

void trim(uchar *s)
{
  /* remove leading and trailing white space */
  uchar *ptr1, *ptr2;
  ptr1 = ptr2 = s;
  while ((*ptr1 == ' ') || (*ptr1 == '\t'))
    ++ptr1;
  while (*ptr1)
    *(ptr2++) = *(ptr1++);
  --ptr2;
  while ((ptr2 > s) && ((*ptr2 == ' ') || (*ptr2 == '\t')))
    --ptr2;
  ++ptr2;
  *ptr2 = 0;
}

/* count appearances of a character in string */
int countc(uchar c,const uchar *s)
{
  int count = 0;
  while (*s)
  {
    if (*s == c)
      ++count;
    ++s;
  }
  return (count);
}

int pos (const uchar *needle,const uchar *haystack)
{
  int plen=strlen((char*)needle);
  int apos=0;
  while (
   (haystack[apos]!=0) &&
   (strncmp((char*)needle,(char*)haystack+apos,plen))
   ) apos++;

  if (haystack[apos]!=0) return apos;
  return -1;
}

static char nbuffer[NEARBUFFERSIZE];

char *Near(char FAR *st)
{
  /* copy FAR string to NEAR nbuffer */
  char *np = nbuffer;
  char FAR *fp = st;
  while ((*fp != 0) && (np - nbuffer < NEARBUFFERSIZE - 1))
    *(np++) = *(fp++);
  *np = 0;
  return (nbuffer);
}
