#ifndef COMP_H
#define COMP_H

#include "modeldef.h"
#include "owbasic.h"

/* lexical analysis */
void putback(void);
#ifdef DEBUGLEX
int lex (void);
#else
int lex1 (void);
#define lex() lex1()
#endif
void progeol (void);
void skipws (void);
#define prognl()  progeol (); ++prog
void expect(int k);
void expect_inumber();
void expect_label();
void tokncpy(uchar *dest,int n);
void tokcpy_ident(uchar *dest);
void sconstncpy(uchar *dest,int n);

/* compiler (parser) functions */
int compile(int lastcode);

void compile_overlay(void);
void compile_procfunc(bool func);
void compile_endp(bool func);
bool compile_userprocedure(uchar *ltype);
int  compile_function(uchar *ltype);
int  compile_command(void);
bool compile_include(bool);
void compile_assignment(void);
void compile_print(void);
void compile_next(void);
void compile_input(void);
void compile_for(void);
void compile_gotoxy(void);
void compile_goto(int jmp);
void compile_if(void);
void compile_else(void);
void compile_endif(void);
void compile_gosub(void);
void compile_return(void);
void compile_exit(void);
void compile_dim(void);
void compile_const(void);
void compile_system(void);
void compile_wait(void);
void compile_case(void);
void compile_of(void);
void compile_endcase(void);
void compile_endwhile(void);
void compile_local (void);
void compile_break (int jmp, int n);
void compile_drop (uchar ltype);
/*
  void compile_repeat(void);
  void compile_until(void);
*/
void compile_while(void);
void compile_do(void);
void compile_loop(void);

void comp_para(uchar FAR *para,uchar *version,int delim);
void comp_var(uchar *dtyp, int* use_addr);
int  comp_varaddr (uchar* dtyp);
void comp_varindex (uchar* ltype);
void comp_array(uchar *dtyp);
void comp_str (void);
void comp_number (void);

int store_label(void);

void serror(int error);
void serror1(int error,const uchar *s);
void serrortoken(int error);

bool comp_exp (uchar wtype);
bool comp_exput (uchar *wtype);

int getint (PC p);
#define getfloat(p) *((FLOATT*) (p.c + 1)) /* float literals as code */
/*#define getfloat(p) *((FLOATT FAR*) (&xdata[*((int*) (p.c + 1))]))*/ /* float literals as fconst() */

int newentry(int dtype,const uchar *n);
int findentry_name(uchar *dtypep,const uchar *n);
int findentry_token(uchar *dtypep);
int searchentry_name(uchar *dtypep,const uchar *n);
int searchentry_token(uchar *dtypep);
int expandentry(int addr,int el,int el_per_line);

void iconst(int i);
void fconst(FLOATT f);

void listreset(void);
void listhide(int pos);
int findvar(int dtype);
int find_label(uchar *s);
int find_token_label(void);
void check_labels(void);

void create_variables(void);

int new_procfunc (uchar *s,PC addr, uchar typ);
int search_procfunc (uchar *s, uchar typ);

int find_eol(void);

extern int listnext;

/*
extern int datanext;  first free "address" after variables (runtime)
extern int firstconst; start of constants (compile and runtime)

defined in owbasic.h
*/

uchar DT(uchar dtyp);

extern uchar FAR default_datatype;

extern bool is_fptrcall;

extern uchar token_type;
extern uchar datatype;
extern int lexint;
extern FLOATT lexfloat;
extern int tok;
extern uchar lex_ref;
extern uchar *token_start;
extern uchar *token_end;
extern uchar *quote;
extern PC apc;

typedef struct keyword
{
  uchar FAR *command;
  int tok;
} Keyword;

#define F_MAXPARAM 9 /* 10 before, 5 minimum (MB,4.22) */
#define F_MAXVERSIONS 3 /* 10 before, 3 minimum (MB,4.22) */

#define P_MAXPARAM 9 /* 10 before, 9 minimum (MB,4.22) */
#define P_MAXVERSIONS 4 /* 10 before, 4 minimum (MB,4.22) */

typedef struct keywordpr
{
  uchar FAR *command;
  uchar paratype[P_MAXPARAM];
  int bincode[P_MAXVERSIONS];
} KeywordPr;

typedef struct keywordfn
{
  uchar FAR *command;
  int rtype;
  uchar paratype[F_MAXPARAM];
  int bincode[F_MAXVERSIONS];
} KeywordFn;

#define MAXPARAM 10

struct proc
{
  uchar name[IDENTLEN];
  uchar paratype[MAXPARAM];
  uchar rtype;
  int len; /* length of code for overlay procedures otherwise 0 */
  PC addr;
};

extern Keyword FAR keyfunctable[];
extern Keyword FAR keytable[];
extern Keyword FAR commandtable[];
/*extern Keyword FAR directivetable[];*/
extern KeywordPr FAR proctable[];
extern KeywordFn FAR functable[];

int look_up(Keyword FAR *list);

extern struct proc FAR proc_table[NUM_PROC];
extern int last_proc;
/*--- include-stack ---*/

typedef struct{
  word fp;
  uchar *prog;
  uchar fname[NAMELEN];
  byte min;
  int line;
  int progtype;
} SourcePtr;

#define INCSTACKSIZE 5 /* TODO: find enough memory to increase stack size (MB,4.10) */
extern SourcePtr IncStack[INCSTACKSIZE];
extern SourcePtr *IncStackPtr;
#define ITOS (*IncStackPtr)

#endif

