#ifndef IBUILD_H
#define IBUILD_H
// Header for interface builder for OWB compiler/interpreter

#include <string>
#include <fstream>
#include <iostream>

using namespace std;

/* datatypes */
#define DUNKNOWN         0
#define DINTEGER         1
#define DFLOAT           2
#define DSTRING          3
#define DLABEL           4
#define DREFERENCE       5
#define DBOOLEAN         6
#define DCHAR            7
#define TYPEMASK         15
/* special kind of data (bit mask) */
#define KREFERENCE       16
#define KLAST            32
#define KARRAY           64
#define KCONST           128

/* kind of entry */
// define then name as string
#define E_NAME 1
#define NAME(x) (x.kind & E_NAME)
// assign an interpreter code
#define E_CODE 2
#define CODE(x) (x.kind & E_CODE)
// insert into compiler tables
#define E_TABLE 4
#define TABLE(x) (x.kind & E_TABLE)
// insert in interpreter table
#define E_INTER 8
#define INTER(x) (x.kind & E_INTER)
// generate interpreter function
#define E_RT 16
#define RT(x) (x.kind & E_RT)

#define MAXPARAM 20
#define MAXFUNC 500
#define MAXCODE 1000

// one entry for list of procedures and functions
typedef struct keywordfn 
{
  int kind;
  string name; // procedure/function name (lower case letters)
  string uname; // procedure/function name (upper case letters)
  char rtype; // return type (functions) or DUNKNOWN for procedures
  char paratype[MAXPARAM]; // parameter types
  string def[MAXPARAM];
  int parnumber; // number of parameters
  int bincode;   // first binary code for this proc/func
  int versions;  // number of versions (different numbers of parameters
  string cname;  // basename interpreterfunction
  string cuname; // basename interpreterfunction (upper case letters)
  string clname; // basename interpreterfunction (lower case letters)
  string condition;
} KeywordFn;

// structure for binary codes
typedef struct bcodeentry {
  int fnr; // pointer to procedure
  bool full;
  int parnumber; // number of parameters (for this version)
} BCodeEntry ;
#endif

